#!/bin/bash
####################################################################
# (C) Copyright 2012-2015 Hewlett-Packard Development Company, L.P.
# @(#) Metrocluster CAEVA Module  Script.
# @(#) Product Name                :  Metrocluster with Continuous Access EVA P6000 for Linux 
# @(#) Product Version             :  B.12.00.30
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################
#
###############################################################################################
#                MULTIPATH RESCAN SCRIPT                                                     
#                -----------------------     
#
#When the package failovers from one node to another node,the permission settings of the LUs 
#in the replication group changes from read-only mode to read-write mode at the destination array. 
#In Linux with the Device Mapper Multipath Software, the hosts do not dynamically detect the 
#associated LU permission change and hence the Metrocluster package start fails. 
#
# As a workaround this script rescans the multipath devices
###############################################################################################

RHEL_DIST=0
#SCSI_ID command path is changed on RHEL7 & SLES12
if [ -e /usr/lib/udev/scsi_id ]; then
        scsi_id_path="/usr/lib/udev"
else
    if [ -f /etc/redhat-release ]; then
      DISTRO="REDHAT"
      scsi_id_path="/lib/udev"
      RHEL_DIST=`cat /etc/redhat-release | awk '{print $7}' | cut -c 1`
    else
      DISTRO="SUSE"
      scsi_id_path="/lib/udev"
    fi
fi
    
	declare -a SCSI_PATH_STR 
	declare -a SCSI_PATH 
	
	wwn=$1
	SCSI_PATH_STR=`multipath -ll $1`
    # Holds the list of EVA/P6000 devices which is not configured with 
    # multipath
#	SCSI_ID_CMD=`lsscsi | grep HSV | grep /dev/ | awk '{print $6}'`
	SCSI_ID_CMD=`lsscsi | grep HSV | grep /dev/ | awk -F/ 'NF>0{print "/dev/"$NF}'`
    # Holds the list of the EVA/P6000 devices configured with multipath
	SCSI_PATH=`echo "${SCSI_PATH_STR}" | grep sd | grep -v "path is down" | sed -n 's/[^0]*0//p' | awk '{print $2}'`

	count=`echo "${SCSI_PATH_STR}" | grep -c "DM multipath kernel driver not loaded"`
	if [[ ($count -ne 0) || ($SCSI_PATH = "") ]]; then
        iteration=0
		for output in ${SCSI_ID_CMD[@]}
                do
    # if the running node is RHEL5 then use the -g -s flag. --whitelisted flag
    # is not supported in RHEL5 - QXCR1001276800
                        if [[ $RHEL_DIST -eq 5 ]]; then
                        output=`echo $output | sed 's/dev/block/g'`
                        temp_wwn=`"$scsi_id_path"/scsi_id -g -s "${output}"`
			else
                        temp_wwn=`"$scsi_id_path"/scsi_id --whitelisted "${output}"`
			fi

                        if [[ "$temp_wwn" = "$wwn" ]]; then
                                if [[ iteration -eq 0 ]];then
                                    echo "Multipathing is not enabled "
                                    echo "Running scan for non multipath devices"
                                    iteration=1
                                fi
                                dev_name=`echo $output | cut -c 6-`
                                echo "Disk rescan started for $dev_name"
                                echo 1 > /sys/block/$dev_name/device/rescan
                                echo "Disk rescan complete for $dev_name"
                        fi
                done
                exit 0
	fi
	count=`echo "${SCSI_PATH_STR}" | grep -c "lib/udev/scsi_id"`
	if [[ $count -ne 0 ]]; then
		echo "multipath -ll command failed to provide expected output"
		echo "The output provided was:"
		echo "${SCSI_PATH_STR}"
		exit 1
	fi

	if [[ "$SCSI_PATH" != "" ]];then
            MPATH_NAME=`echo "${SCSI_PATH_STR}" | grep $1 | awk '{print $1}'` 

# This code works for user friendly names,aliases,wwid configurationi - QXCR1001276803
	
	if [[ "$MPATH_NAME" != "" ]];then
	    echo "Multipathing is enabled"
	    echo "Running scan for multipath device ${MPATH_NAME}"
	    multipath -f ${MPATH_NAME}
        fi
		for output in ${SCSI_PATH[@]}
		do
			echo "Disk rescan started for $output"
			echo 1 > /sys/block/$output/device/rescan
			echo "Disk rescan complete for $output"
		done

		    multipath -v0
	fi

