####################################################################
# (C) Copyright 2012-2016 Hewlett Packard Enterprise Development LP. 
# @(#) Metrocluster CAEVA Module  Script.
# @(#) Product Name                :  Metrocluster with Continuous Access EVA P6000 for Linux 
# @(#) Product Version             :  B.12.00.51
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################



if [ -f /etc/redhat-release ]; then
  DISTRO="REDHAT"
  CMCLUSTER="/usr/local/cmcluster"
else
  DISTRO="SUSE"
  CMCLUSTER="/opt/cmcluster"
fi

CMGETPKGENV=${CMCLUSTER}/bin/cmgetpkgenv
#
#
#SG_UTILS=/usr/local/cmcluster/conf/scripts/mscripts/utils.sh
SG_UTILS=${CMCLUSTER}/conf/scripts/mscripts/utils.sh
export SG_UTILS

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

#########################################
# Source Metrocluster  utility functions.
##########################################

#MC_UTILS=/usr/local/cmcluster/conf/scripts/dts/caeva_util.sh
MC_UTILS=${CMCLUSTER}/conf/scripts/dts/caeva_util.sh
export MC_UTILS

if [[ -f ${MC_UTILS} ]]; then
    . ${MC_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package Metrocluster utility functions file: ${MC_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find Metrocluster utility functions file: ${MC_UTILS}"
    exit 1
fi

#################################
# Source hpux internal functions.
#################################

#if [[ -f $SG_INTERNAL_FUNCS_FILE ]] ; then
#    . $SG_INTERNAL_FUNCS_FILE
#    if (( $? != 0 )) ; then
#        sg_log 0 "ERROR: Unable to source SG internal functions file: $SG_INTERNAL_FUNCS_FILE"
#        exit 1
#    fi
#else
#    sg_log 0 "ERROR: Unable to find SG internal functions file: $SG_INTERNAL_FUNCS_FILE"
#    exit 1
#fi
###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

# Storing cmviewcl command output.
cmviewcl_output_mc=`cmviewcl -vf line -s config`

###############################################################################
# This function invokes the metrocluster caeva binary
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - metrocluster caeva operation is successful
#         exit - 0 - metrocluster caeva operation is not successful
#
###############################################################################

function runmccaeva
{
	typeset swap_status;
	typeset ret;

	declare -a site1_nodes
	declare -a site2_nodes
	declare -a nodes
	
	if [[ -f ${CMCLUSTER}/bin/DRCheckCAEVADevGrp ]];then
 			DRCheckCAEVADevGrp $1 0 0 "" ""
			swap_status=$?
	else
        echo "ERROR: Cannot find the file: DRCheckCAEVADevGrp file. Exiting."
	exit 1
	fi

    if [[ $swap_status -eq 0 || $swap_status -eq 3 ]];then
# Multipath script should run regardless of the personality switch of the dr group 
# QXCR1001277896 
        storage_name=""
        site_number=0
        site1_nodes=`$CMGETPKGENV ${SG_PACKAGE_NAME} | grep CAEVA_DC1_HOST_LIST | cut -d= -f2 | tr ',' '\n' | tr '"' ' ' | cut -d. -f1`
        site2_nodes=`$CMGETPKGENV ${SG_PACKAGE_NAME} | grep CAEVA_DC2_HOST_LIST | cut -d= -f2 | tr ',' '\n' | tr '"' ' ' | cut -d. -f1`
        current_node=`uname -n | awk -F'.' '{print $1}'`
        for system in ${CAEVA_DC1_HOST_LIST}; do
            systemWithoutDomain=$(echo $system | sed 's/\"//g' | awk -F"." '{print $1}');
            if [[ ${systemWithoutDomain}  = ${current_node} ]]; then
                site_number=1
            storage_name=`echo $CAEVA_DC1_STORAGE_WORLD_WIDE_NAME | awk -F'-' '{print $1$2$3$4}'`
            fi
        done
        for system in ${CAEVA_DC2_HOST_LIST}; do
            systemWithoutDomain=$(echo $system | sed 's/\"//g' | awk -F"." '{print $1}');
            if [[ ${systemWithoutDomain}  = ${current_node} ]]; then
                site_number=2
            storage_name=`echo $CAEVA_DC2_STORAGE_WORLD_WIDE_NAME | awk -F'-' '{print $1$2$3$4}'`
            fi
        done

        checkifmodused "dts/cccaeva"
        ret=$?
        if [ $ret -eq 0 ]
        then
                sg_log 5 "Continentalclusters  detected "
                if [[ $site_number -eq 1 ]]; then
                        nodes=`echo ${site1_nodes[@]}`
                elif [[ $site_number -eq 2 ]]; then
                        nodes=`echo ${site2_nodes[@]}`
                fi
        else
                        nodes=`echo ${site1_nodes[@]};echo ${site2_nodes[@]}`
        fi

        if [[ -f ${CMCLUSTER}/bin/evadiscovery ]];then
            typeset vdisks=""
            OLDIFS=$IFS
            IFS=$'\n'
            foundArray=1
            foundDRGrp=1
            temp=`evadiscovery -l`
            if [[ $? -eq 0 ]];then
                for line in ${temp[@]}; do
                    if [[ $foundDRGrp -eq 0 ]];then
                        echo $line | grep "WWID List:" 2>&1>/dev/null
                        if [[ $? -eq 0 ]];then
                            continue
                        fi
                        echo $line | grep -e "DR Group Name:" -e "Storage WWN:" 2>&1>/dev/null
                        if [[ $? -eq 0  ]];then
                            break
                        else
                            trimmedline=`echo ${line}|tr -d ' '`
                            vdisks=("${vdisks[@]}" "${trimmedline}")
                        fi
                     fi
                     if [[ $foundArray -eq 0 ]];then
                        echo $line | grep -w "DR Group Name: \\\\Data Replication\\\\${CAEVA_DR_GROUP_NAME}" 2>&1>/dev/null
                        if [[ $? -eq 0 ]];then
                            foundDRGrp=0
                            continue
                        fi
                     fi
                     if [ "$line" == "Storage WWN: ${storage_name}" ];then
                        foundArray=0
                     fi
                done
            fi
            IFS=$OLDIFS

            for disk in ${vdisks[@]}; do
                disk=`echo $disk | awk '{print tolower($0)}'`
                rescan_output=`${CMCLUSTER}/bin/mccaeva/mccaeva_multipath_rescan.sh 3${disk}`
                sg_log 0 "${rescan_output}"
                LoopCount=0
                ScanOutFile=$CAEVA_DTS_PKG_DIR/${SG_PACKAGE_NAME}_DeviceScanStatus
                for out in ${nodes[@]}; do
                    if [[ "$out" !=  "$current_node" ]]; then
                        if [[ LoopCount -eq 0 ]]; then
                            cat /dev/null > $ScanOutFile
                            LoopCount=1
                        fi
                        echo "Running Multipath rescan script on node $out"
                        echo "Running Multipath rescan script on node $out" >> $ScanOutFile
                        cmexec $out -o $ScanOutFile.tmp ${CMCLUSTER}/bin/mccaeva/mccaeva_multipath_rescan.sh 3${disk}
                        cat $ScanOutFile.tmp >> $ScanOutFile
                    fi
                    rm -f $ScanOutFile.tmp
                done
            done
        fi
        sleep 10 # sleeping for rescan to take effect

        if [[ $swap_status -eq 3 ]];then # Storage switched from read-only to read-write
                echo "export SFC_MC_RESCAN_REQUIRED=\"1\"" >$SG_SRC_FRM_CHILD
                for vg in ${SG_VG[*]};do
                            typeset hostid=""
                            hostid=$(vgs --noheadings -o +tags $vg | awk  '{print $8}')
                            if [[ "$hostid" = *,* ]]; then
                                sg_log 0 "ERROR: Volume Group $vg has multiple tags \"($hostid)\" defined."
                                sg_log 0 "There cannot be more than one tag."
                                sg_log 0 "Remove the tags using the command:"
                                for i in ${hostid}; do
                                    sg_log 0 "\"vgchange --deltag $i $vg\"."
                                done
                                return 1
                            fi

                            if [[ "$hostid" != "" ]] ; then
                            #       cmexec $hostid vgchange -a n $vg
                                    vgchange --deltag $hostid $vg
                            fi
                    done
                    swap_status=0
        fi
    fi
	if [[ $swap_status -ne 0 ]];then
		exit $swap_status
	fi
	return 0
}

###############################################################################
# This function invokes the metrocluster caeva binary
# Internal Use only
#
# Arguments:  No Arguments
#
# Returns:   WWID list of Disk group
#
###############################################################################


function get_dr_wwids
{
        storage_name=`echo $CAEVA_DC1_STORAGE_WORLD_WIDE_NAME | awk -F'-' '{print $1$2$3$4}'`

        if [[ -f ${CMCLUSTER}/bin/evadiscovery ]];then
            OLDIFS=$IFS
            IFS=$'\n'
            foundArray=1
            foundDRGrp=1
            temp=`evadiscovery -l`
            if [[ $? -eq 0 ]];then
                for line in ${temp[@]}; do
                    if [[ $foundDRGrp -eq 0 ]];then
                        echo $line | grep "WWID List:" 2>&1>/dev/null
                        if [[ $? -eq 0 ]];then
                            continue
                        fi
                        echo $line | grep -e "DR Group Name:" -e "Storage WWN:" 2>&1>/dev/null
                        if [[ $? -eq 0  ]];then
                            break
                        else
                            trimmedline=`echo ${line}|tr -d ' '`
                            vdisks=("${vdisks[@]}" "${trimmedline}")
                        fi
                     fi
                     if [[ $foundArray -eq 0 ]];then
                        echo $line | grep -w "DR Group Name: \\\\Data Replication\\\\${CAEVA_DR_GROUP_NAME}" 2>&1>/dev/null
                        if [[ $? -eq 0 ]];then
                            foundDRGrp=0
                            continue
                        fi
                     fi

                     if [ "$line" == "Storage WWN: ${storage_name}" ];then
                        foundArray=0
                     fi
                done
            fi
            IFS=$OLDIFS
        fi
}

#######################################################################
#
# function sg_start_caeva() is called by master control script
# during metrocluster caeva package startup.
#
#######################################################################


function sg_start_caeva
{
	typeset envfile;
	typeset count;
	typeset ret;
	typeset version=0;

	sg_log 0 "Starting Metrocluster CAEVA Package"

	checkifmodused "dts/sc"
        ret=$?
        	if [ $ret -eq 0 ]
        	then
        		sg_log 0 "Site Controller module detected "
        		exit 0
        	fi

        find_module_version "dts/caeva"
        version=$?

        if [[ $version -eq 1 ]]; then

                runmccaeva $SG_PACKAGE_NAME
                to_exit=0
        else	
				envfile=`/bin/ls  $CAEVA_DTS_PKG_DIR/*.env  2>&1 | grep -v "not found" `
				if [ "X${envfile}" =   "X"  ]
				then
					echo "ERROR: Metrocluster environment file not found for package $SG_PACKAGE under $CAEVA_DTS_PKG_DIR on the node"
					to_exit=1
					exit $to_exit
				fi
					count=0;
				for i in $envfile
				do
					count=`expr $count + 1`
				done
				if [[ $count != 1 ]]
				then
					echo "ERROR: More than one environment file found under $CAEVA_DTS_PKG_DIR directory"
					to_exit=1;
					exit $to_exit;
				fi
			runmccaeva $envfile
			to_exit=0
		fi
}

#######################################################################
#
# function sg_stop_caeva() is called by master control script
# during metrocluster caeva package stop.
#
#######################################################################


function sg_stop_caeva
{
    sg_log 0 "Stopping Metrocluster CAEVA Package"
    # no op
    to_exit=0
}

#######################################################################
#
# function caeva_env_gen() is called by sg_validate_caeva
# during metrocluster caeva configuration application.
#
#######################################################################
function caeva_env_gen
{
	env_file_location="${CAEVA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_caeva.env"
echo "
# **********************************************************************
# *        Environment file for Continuous Access EVA                  *
# *                                                                    *
# *            This file is AUTO-GENERATED.	                       *
# *                                                                    *
# **********************************************************************
#
# INFORMATION:  Please read the MC/CAEVA Release Notes found at
# http://docs.hp.com/hpux/ha/index.html#MetroCluster
#
# PURPOSE:  This environment file will be used by the CA EVA toolkit
# to check the status of the Data Replication group that is used by
# the package.
#
# LOCATION: This environment file has to be placed in the package 
# directory.
#
# NAMING: This environment file has to be named with the combination of
# the package control script file name without .cntl extension and 
# concatenate with _caeva.env.  For example, if the name of the package
# control script is pkgA.cntl then the name of this environment file 
# would be pkgA_caeva.env.
#
# The following variables must be customized for each package.
#
# CLUSTER_TYPE
#	This parameter should be set to \"metro\" if this is a Metrocluster
#	and \"continental\" if this is a Continental Cluster. The default 
#	is \"metro\". CLUSTER_TYPE=metro is supported only when the HP
#	Metrocluster product is installed. CLUSTER_TYPE=continental is 
#	supported only when the HP ContinentalClusters product has been
#	installed.
#
# PKGDIR
#	Contains the full path name of the package directory. This 
#	directory must be unique for each package. The operator may create
#	the FORCEFLAG file in this directory.
#
# DT_APPLICATION_STARTUP_POLICY
#	This parameter defines the preferred policy in allowing the 
#	application to start with respect to the state of the data in the
#	local volumes. It should be set to one of the following two 
#	policies:
#
#	Availability_Preferred:
#		Choose this policy if you preferxs application 
#		availability. Metrocluster software will allow the application
#		to start as long as the data is consistent; even though, it 
#		may not be current.
#
#	Data_Currency_Preferred:
#		Choose this policy if you prefers the application 
#		to operate on consistent and current data. Metrocluster
#		software will only allow the application to start if it can 
#		tell for sure that the data the application will operate on
#		is current. This policy only focuses on the state of the local
#		data (with respect to the application) being consistent and current.
#
# WAIT_TIME
#	0 or greater than 0 (in minutes)
#	This parameter defines the timeout, in minutes, to wait for 
#	completion of the data merging or copying for the DR group before 
#	startup the package on destination volume. The default wait time 
#	is 0 minute (means no wait). If WAIT_TIME expires and merging or 
#	copying is still in progress, the package will fail to start with
#	an error not to starting on any node in the cluster.
#
# DR_GROUP_NAME
#	The name of DR Group used by this package. This DR Group name is
#	defined when the DR Group is created. The name should be in between 
#	double quotes.
#
# DC1_STORAGE_WORLD_WIDE_NAME
#	The World Wide Name of EVA storage system resides in Data Center 1.
#	This storage system name is defined when the storage is initialized.
#
# DC1_SMIS_LIST
#	A list of Management Server resides in Data Center 1. Multiple names can
#	be defined using comma as separator between the names. The name 
#	list should be in between double quotes. 
#
# DC1_HOST_LIST
#	A list of clustered nodes resides in Data Center 1.  Multiple names
#	can be defined using comma as separator between the names. The name 
#       list should be in between double quotes.
#
# DC2_STORAGE_WORLD_WIDE_NAME
#	The World Wide Name of EVA storage system resides in Data Center 2.
#	This storage system name is defined when the storage is initialized.
#
# DC2_SMIS_LIST
#	A list of Management Server resides in Data Center 2. Multiple names can
#	 be defined using comma as separator between the names. The name list
#	should be in between double quotes.
#
# DC2_HOST_LIST
#	A list of clustered nodes resides in Data Center 2.  Multiple names can
#	be defined using comma as separator between the names.  The name 
#       list should be in between double quotes. 
#
# QUERY_TIME_OUT
#	Sets the time out in seconds waiting for a response from the SMI-S server.
#	The default time out is 300 seconds.
#
#
# Example:
#
#	CLUSTER_TYPE=metro
#	PKGDIR=/etc/cmcluster/pkgA
#	DT_APPLICATION_STARTUP_POLICY=Data_Currency_Preferred
#	WAIT_TIME=5
#	DR_GROUP_NAME=\"DR Group 1\"
#	DC1_STORAGE_WORLD_WIDE_NAME=5000-1FE1-5000-DDE0
#	DC1_SMIS_LIST=\"128.12.15.51, 128,12,15,52\"
#	DC1_HOST_LIST=\"dts1, dts2\"
#	DC2_STORAGE_WORLD_WIDE_NAME=5000-1FE1-5000-DDF0 
#	DC2_SMIS_LIST=\"128.12.16.57, 128,12,16,582\"
#	DC2_HOST_LIST=\"dts3, dts4\"
#	QUERY_TIME_OUT=300
DT_APPLICATION_STARTUP_POLICY=$CAEVA_DT_APPLICATION_STARTUP_POLICY
WAIT_TIME=$CAEVA_WAIT_TIME
DR_GROUP_NAME=$CAEVA_DR_GROUP_NAME
DC1_STORAGE_WORLD_WIDE_NAME=$CAEVA_DC1_STORAGE_WORLD_WIDE_NAME
DC1_SMIS_LIST=$CAEVA_DC1_SMIS_LIST
DC1_HOST_LIST=$CAEVA_DC1_HOST_LIST
DC2_STORAGE_WORLD_WIDE_NAME=$CAEVA_DC2_STORAGE_WORLD_WIDE_NAME
DC2_SMIS_LIST=$CAEVA_DC2_SMIS_LIST
DC2_HOST_LIST=$CAEVA_DC2_HOST_LIST
QUERY_TIME_OUT=$CAEVA_QUERY_TIME_OUT
PKGDIR=$CAEVA_DTS_PKG_DIR
CLUSTER_TYPE=$CAEVA_CLUSTER_TYPE" > ${env_file_location}

chmod 755 ${env_file_location}
return $?

}

#######################################################################
#
# function sg_verify_eva_pkg is called by sg_validate_eva()
# during metrocluster eva configuration validation.
#
#######################################################################

function sg_verify_eva_pkg
{
	typeset DC1_drstate=""
    CMGETPKGENV=${CMCLUSTER}/bin/cmgetpkgenv
	IOSCAN=/usr/sbin/ioscan

#step1: Verify the values specified for Metrocluster
#        module parameters are valid and validates
#        whether the node can connect to the SMIS servers
#	This way we can check whether array management 
#	Software is accessible from the node.
#	cmapplyconf will anyway generate environment file. 
#	The environment file however needs to be generated 
#	during cmcheckconf -P/-p with a temporary name to validate 
#	the inputs in the package configuration file.
#
#Always generate the env file with a temporary name. Delete the generated env file
#after cmcheckconf and cmapplyconf operations

		caeva_env_gen
		internal_use=1
		to_exit=$?
	
	if [[ "$SG_COMMAND" = "cmcheckconf" ]]; then
                cmdVal="cmcheckconf"
        else
                cmdVal="cmapplyconf"
        fi

		envFile=${CAEVA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_caeva.env

		${CMCLUSTER}/bin/mccaeva/DREVAValidateAttrib $envFile $cmdVal

	if [[ $? -ne 0 ]]; then
		errors[$errcount]="Error: Validation of Metrocluster attributes failed.\n"
		(( errcount=errcount+1 ))
	fi

    if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
             verify_vmfs_details
             if [ $? -ne 0 ]; then
                 echo "ERROR: Failed to validate VMFS details mentioned in the package"
                 to_exit=1
             fi
    else 
    #Step 2 : Verify if all disks used by a Metrocluster
    #        package are replicated

        checklvmused ${SG_VG[*]}
        ret=$?
        if (( $ret == 0 )) || (( $ret == 2 )) ; then
            list_of_vgs="${SG_VG[@]}"
            lvm_disks_replicated  
        fi

        vxvmused=0
        checkvxvmused ${SG_VXVM_DG[*]}
        if [ $? -ne 1 ]; then
            vxvmused=1
        fi

        checkcvmused ${SG_CVM_DG[*]}
        if [ $? -ne 1 ]; then
            vxvmused=1
        fi

        if [[ ${vxvmused} -eq 1 ]]; then
            # no separate treatment for cvm and vxvm disk
            # merge the dgnames into a common array.
            list_of_dgs="${SG_VXVM_DG[@]} ${SG_CVM_DG[@]}"
            vxvm_disks_replicated 
        fi
    fi

	#Delete the temporary file which has SMI-S servers list
 	if [[ -f ${envFile}.smis ]];then
		rm ${envFile}.smis
        fi

	#Delete the temporary env file generated internally
	if [[ "${internal_use}" -eq 1 ]]; then
		rm -f ${CAEVA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_caeva.env
	fi

	i=0
	while [ $i -lt ${#errors[@]} ]; do
		echo ${errors[$i]}
		(( i = i + 1 ))
	done

	i=0
	while [ $i -lt ${#warnings[@]} ]; do
		echo ${warnings[$i]}
		(( i = i + 1 ))
	done

	if  [[ ${errcount} -gt 0 ]]; then
		return 1
	else
		return 0
	fi

}

###############################################################################
# This function checks details about VMFS disks specified.
# Internal Use only
#
# Returns: 0 if SUCCESS
#          1 if disk specified in package is not part of RCVG
###############################################################################
function verify_vmfs_details
{
    typeset eligible_nodes_vmfs=""
    typeset ro_nodes=""
    typeset wwids_rw=""
    typeset wwids_ro=""
    typeset rcvg_rw=""
    typeset rcvg_ro=""
    typeset RW_site=$1
    typeset -i disk_flag=0
    typeset vdisks=""
    typeset local_storage_wwid=""

    cur_hostname=${SG_HOST_NAME%%.*}
    #echo "${CAEVA_DC1_HOST_LIST}" | grep "${cur_hostname}" >/dev/null 2>&1
    #if [[ $? -eq 0 ]]; then
    #    local_storage_wwid=$DC1_STORAGE_WORLD_WIDE_NAME 
    #else
    #    local_storage_wwid=$DC2_STORAGE_WORLD_WIDE_NAME 
    #fi 

    #Find RW and RO node list
    dc1_smislist_space=`echo ${CAEVA_DC1_SMIS_LIST} | sed 's/,/ /g'`
    DC1_drstate=`${CMCLUSTER}/bin/mccaeva/DREVAGetDRState $CAEVA_DR_GROUP_NAME \
    ${CAEVA_DC1_STORAGE_WORLD_WIDE_NAME} $dc1_smislist_space`
    echo $DC1_drstate | grep "participationType: source" >/dev/null 2>&1
    if [[ $? -eq 0 ]]; then
         eligible_nodes_vmfs=(${CAEVA_DC1_HOST_LIST})
         ro_nodes=(${CAEVA_DC2_HOST_LIST})
    else
         eligible_nodes_vmfs=(${CAEVA_DC2_HOST_LIST})
         ro_nodes=(${CAEVA_DC1_HOST_LIST})
    fi

    # Pass the eligible nodelist to Master Control Script
    echo "export SFC_MC_READ_WRITE_NODELIST=\"${eligible_nodes_vmfs[*]}\"" >$SG_SRC_FRM_CHILD
    echo "export SFC_MC_READ_ONLY_NODELIST=\"${ro_nodes[*]}\"" >>$SG_SRC_FRM_CHILD

    # Verification is done only from one node.
    if [[ ${cur_hostname} = ${eligible_nodes_vmfs[0]} ]]; then
        eligible_node_uuid=$(get_uuid_from_node_name ${eligible_nodes_vmfs[0]})
        ro_node_uuid=$(get_uuid_from_node_name ${ro_nodes[0]})
        vmfs_details=$($SGSBIN/cmvmdiskquery -n $eligible_node_uuid -n $ro_node_uuid)
        if [[ $? -ne 0 ]]; then
                echo "ERROR: Unable to query the ESX Host or VCenter"
                return 1
        fi

        # Verifying the RW site.
        get_dr_wwids

        # Verifying that all the disks in the DR Group is part of the Package.
        for disk_wwid in ${vdisks[@]} 
        do
            disk_flag=0
            for ds_name in ${SG_DATASTORE_NAME[*]}
            do
                echo "$vmfs_details"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|grep -i "$disk_wwid" >/dev/null 2>&1
                if [[ $? -eq 0 ]]; then
                    disk_flag=1
                fi
            done
            if [[ $disk_flag -eq 0 ]];then
                echo "WARNING: The disk with WWID $disk_wwid, configured as part of the Data Replication group $CAEVA_DR_GROUP_NAME"
                echo "         is not part of any of the Datastore specified in the package."
            fi
        done

        # Verifying that all the disk in the Data Store is part of DR group 
        disk_flag=0
        for ds_name in ${SG_DATASTORE_NAME[*]}
        do
            ds_wwids=`echo "${vmfs_details}"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|cut -d '=' -f2`
            for wwid in ${ds_wwids}
            do
                echo "${vdisks[@]}"|grep -i "${wwid##*.}" >/dev/null 2>&1
                if [[ $? -ne 0 ]]; then
                    echo "ERROR: The disk with WWID ${wwid##*.}, used for the Datastore ${ds_name} is not part of the Data Replication group $CAEVA_DR_GROUP_NAME"
                    disk_flag=1
                fi
            done
        done
        if [[ $disk_flag -eq 1 ]];then
            return 1
        fi


        # Verifying that all the disks in the RCVG is part of the Package.
        for disk_wwid in ${vdisks[@]} 
        do
            echo "$vmfs_details"|grep -i "VirtualMachine:${ro_node_uuid}|Disk:" |grep -i $disk_wwid >/dev/null 2>&1
            if [[ $? -ne 0 ]]; then
                echo "WARNING: The disk with WWID $disk_wwid, configured as part of the Data Replication group $CAEVA_DR_GROUP_NAME"
                echo "         in the Read only site is not available to the host."
            fi
        done
    fi
}


#######################################################################
#
# function sg_validate_caeva() is called by master control script
# during metrocluster caeva configuration validation.
#
#######################################################################

function sg_validate_caeva
{
	#no op
	typeset version=0;
	find_module_version "dts/mccaeva"
	version=$?

	validate_mc
	if [[ $? -gt 1 ]]; then
		to_exit=1
		exit $to_exit
	fi
        sg_major_version=`${CMCLUSTER}/bin/cmversion | awk -F'.' '{ print $2 }'`
        sg_minor_version=`${CMCLUSTER}/bin/cmversion | awk -F'.' '{ print $3 }'`

		
	if [[ $version -eq 1 ]]; then
	
		if [[ ${sg_major_version} -ge 12 ]]; then
			sg_verify_eva_pkg	
			to_exit=$?
		elif [[ ${sg_major_version} -eq 11 && ${sg_minor_version} -ge 20 ]]; then
			sg_verify_eva_pkg
			to_exit=$?
		fi
	fi

	
}
################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#


typeset to_exit=0;



case ${1} in
       start)      sg_start_caeva;;
        stop)      sg_stop_caeva;;
    validate)    sg_validate_caeva ;;
esac
#
# Exit to indicate success/failure to the master_control_script.sh
#

exit $to_exit


