####################################################################
# (C) Copyright 2011-2016 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Metrocluster for Linux
# @(#) Product Version             :  B.12.00.50
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release HP Metrocluster
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
#
####################################################################


if [ -f /etc/redhat-release ]; then
 DISTRO="REDHAT"
 CMCLUSTER="/usr/local/cmcluster"
else
 DISTRO="SUSE"
 CMCLUSTER="/opt/cmcluster"
fi

SG_UTILS=${CMCLUSTER}/conf/scripts/mscripts/utils.sh
export SG_UTILS

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
   . ${SG_UTILS}
   if (( $? != 0 ))
   then
      echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
      exit 1
   fi
else
   echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
   exit 1
fi
###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

#######################################################################
#
# function sg_start_mc() is called by master control script
# during metrocluster package startup.
#
#######################################################################


function sg_start_mc
{
	sg_log 0 "Starting Metrocluster"
    	# no op
    	to_exit=0


}
#######################################################################
#
# function sg_stop_mc() is called by master control script
# during metrocluster package stop.
#
#######################################################################

function sg_stop_mc
{
	sg_log 0 "Stopping Metrocluster"
    	# no op
	to_exit=0
}
#######################################################################
#
# function sg_validate_mc() is called by master control script
# during metrocluster package configuration validation.
#
#######################################################################

function sg_validate_mc
{
        to_exit=0
	exit $to_exit


}

declare -a errors
declare -a warnings
declare -a list_of_dgs
declare -a list_of_vgs

errcount=0
warncount=0
internal_use=0

###############################################################################
# This function verifies whether the SG-CFS package is running in the cluster.
# Internal Use only
#
#
# Arguments: No arguments
#
#
#
# Returns:    0 - the SG-CFS Package is running in the cluster
#             exits if the SG-CFS Package is not running in the cluster
#
###############################################################################


function checkSGCFS
{
   typeset cfspkgstatus;
   cfspkgstatus=`cmviewcl -f line -p SG-CFS-pkg | awk -F'=' '$1 == "status" {print $2}'`
   if [ $? != 0 ]
   then
      sg_log 0 "\tERROR: Function checkSGCFS"
      sg_log 0 "\tERROR: Failed to get the SG-CFS-pkg system multi-node package status"
      exit 1
   fi
   if [ "$cfspkgstatus" != "up" ]
   then
      sg_log 0 "\tERROR: Function SGCFS"
      sg_log 0 "\tERROR: The SG-CFS-pkg system multi-node package is not up"
      exit 1
   fi
   sg_log 0 "\tINFO: SG-CFS-pkg system multi-node package is up"
   return 0
   
}

##################################################################################
# This function checks whether the volume manager is cvm
# Internal Use only
#
#
# Arguments: No arguments

#
#
# Returns:  0 - cvm is used
#           1 - cvm is not used
#
###############################################################################



function checkcvmused
{
   
   if [[ ${SG_CVM_DG[*]} != "" ]]
   then
      
      if [[ ${SG_CVM_ACTIVATION_CMD} = "" ]]
      then
         sg_log 0 "\tERROR: Function checkcvmused"
         sg_log 0 "\tERROR: CVM Activation Command can not be empty while using CVM group"
         exit 1
      fi
      checkSGCFS
      if [ $? -ne 1 ]
      then
         return 0
      fi
   fi
   return 1
}
###############################################################################
# This function checks whether the volume manager is vxvm
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - vxvm is used
#          1 - vxvm is not used
#
###############################################################################


function  checkvxvmused
{
   if [[ ${SG_VXVM_DG[*]} != "" ]]
   then
      return 0
   fi
   return 1
}

###############################################################################
# This function checks whether the volume manager is lvm or slvm
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - lvm is used
#          1 - lvm is not used
#          2 - slvm is used
#
###############################################################################

function checklvmused
{
   if [[ ${SG_VG[*]} != "" ]]
   then
      
      if [[ ${SG_VGCHANGE_CMD} = "" ]]
      then
         sg_log 0 "\tERROR: Function checklvmused"
         sg_log 0 "\tERROR: LVM Activation Command can not be empty while using LVM group"
         exit 1
      fi
      
      if [[ "${SG_VGCHANGE_CMD#*-a}" = " s" ]]
      then
         return 2;
      fi
      return 0
   fi
   return 1
}



###############################################################################
# This function checks whether given module is included or not
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - module is used
#          1 - module is not used
#
###############################################################################

function checkifmodused
{
   typeset used_module=$1;
   let exit_val=1
   typeset module_name;
   typeset module;
   
   module_name=`echo "${SG_MODULE_NAME[@]}"`
   
   for module in $module_name
   do
      if [ "$module" = "$used_module" ]
      then
         let exit_val=0
         break
      fi
   done
   
   return $exit_val
   
}

###############################################################################
# This function returns the version of the given module
# Internal Use only
#
#
# Arguments: The name of the module
#
#
#
# Returns: The version of the module
#
###############################################################################

function find_module_version
{
   typeset used_module=$1;
   typeset module_name;
   let counter=0
   typeset module;
   typeset module_location;
   module_name=`echo "${SG_MODULE_NAME[@]}"`
   
   for module in $module_name
   do
      if [ "$module" = "$used_module" ]
      then
         module_location=$counter
         break
      else
         let counter=counter+1
      fi
   done
   return ${SG_MODULE_VERSION[${module_location}]}
   
   
}


###############################################################################
# This function returns the number of environemnt files present in the package
# directory
#
# Internal Use only
#
#
# Arguments: No arguments
#
#
#
# Returns: The count of environment files in the package directory
#
###############################################################################

function find_env_count
{
   typeset envfile;
   typeset count;
   
   envfile=`/bin/ls  $CAEVA_DTS_PKG_DIR/*.env  2>&1| grep -v "not found" `
   if [ $? -eq 0 ]
   then
      count=0;
      for i in $envfile
      do
         count=`expr $count + 1`
      done
   fi
   return ${count}
   
}

###############################################################################
# This function checks validates the Metrocluster configuration
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - Validation succeeded
#          1 - Validation failed
#
###############################################################################

function validate_mc
{
   
   typeset envfile;
   typeset samefile;
   typeset count;
   
   sg_log 0 "Validating Metrocluster Configuration"
   
   let version=0
   
   find_module_version "dts/caeva"
   version=$?
    
   if [ $version -eq 1 ]; then
	
      if [[ ! -d $CAEVA_DTS_PKG_DIR ]]; then
         mkdir -p -m 755 $CAEVA_DTS_PKG_DIR
         if [ $? -eq 0 ]; then
            sg_log 0 "Creation of \"$CAEVA_DTS_PKG_DIR\" successful"
         else
            echo "ERROR:Failed to create \"$CAEVA_DTS_PKG_DIR\" "
            return 1
         fi
      fi
   fi
   
   if [[ ! -d $CAEVA_DTS_PKG_DIR ]]
   then
      echo "ERROR:  \"$CAEVA_DTS_PKG_DIR\" directory does not exist"
      to_exit=1
      return $to_exit
   fi
   
   find_env_count
   count=$?
   
   if [[ $count -eq 0 && $version -eq 0 ]]
   then
      echo "ERROR: Metrocluster environment file not found for package \"$SG_PACKAGE_NAME\" under
      the directory \"$CAEVA_DTS_PKG_DIR\""
      to_exit=1;
      return $to_exit;
   fi
   
   if [[ $count -gt 1 && $version -eq 0 ]]
   then
      echo "ERROR: More than one environment file found under \"$CAEVA_DTS_PKG_DIR\" directory"
      to_exit=1;
      return $to_exit;
   fi
   to_exit=0
   
   return $to_exit
   
}





##################################################################################
# This function checks whether the disks belonging to a LVM or SLVM are being
# replicated.
#
# Internal Use only
#
#
# Arguments: None
#
#
# Returns: None 
#
###############################################################################


function lvm_disks_replicated
{
    LVMADM=/usr/sbin/lvmadm
    IOSCAN=/usr/sbin/ioscan

    for vg in ${list_of_vgs[@]} ; do
       # retrieve the device info from lvmadm command
       list_of_devices=$($LVMADM -l | awk -v vg=/dev/${vg} -v vgfound=0 '
                                        /^VG Name/ {
                                                if ( $3 == vg )
                                                        vgfound = 1
                                                else
                                                        vgfound = 0
                                                }

                                        /^PV Name/ {
                                                if ( vgfound == 1 )
                                                        print $3
                                                }

                                        NF == 1 && /\/dev\/(dsk|disk|cdisk)\// {
                                                if ( vgfound == 1 )
                                                        print $1
                                        }')

        unset xpca_list_of_devices
        for device in ${list_of_devices}; do
            # get the persistent dsf equivalent only if the device is in legacy dsf or cluster dsf
            if [[ $device = /dev/dsk* ]]; then
                disk_in_pers_dsf=`${IOSCAN} -m dsf ${device} | grep ${device} | awk '{ print $1 }'`
            elif [[ $device = /dev/cdisk* ]]; then
                disk_in_pers_dsf=`${IOSCAN} -m cluster_dsf ${device} | grep ${device} | awk '{ print $2 }'`
            else
                disk_in_pers_dsf=${device}    
            fi
            disk_in_pers_dsf=`echo $disk_in_pers_dsf | sed 's/\(^\/...\/\)/\1r/'`

            # call the appropriate function based on the module used.
            checkifmodused "dts/srdf"
            if [ $? -eq 0 ]; then
                srdf_get_disk_info ${disk_in_pers_dsf} ${vg} "lvm"
            fi

            checkifmodused "dts/xpca"
            if [ $? -eq 0 ]; then
                xpca_list_of_devices[${#xpca_list_of_devices[@]}]=$disk_in_pers_dsf
            fi

            checkifmodused "dts/caeva"
            if [ $? -eq 0 ]; then
                caeva_get_disk_info ${disk_in_pers_dsf} ${vg} "lvm"
            fi
        done
        checkifmodused "dts/xpca"
        if [ $? -eq 0 ]; then
            xpca_get_disk_info ${vg} "lvm"
        fi
    done
}


##################################################################################
# This function checks whether the disks belonging to a VXVM or CVM diskgroup are
# being replicated.
#
# Internal Use only
#
#
# Arguments: None
#
#
# Returns: None 
#
###############################################################################

function vxvm_disks_replicated
{
    IOSCAN=/usr/sbin/ioscan

    for vg in ${list_of_dgs[@]}; do
        list_of_devices=$(vxdisk -s list | awk -v vg=$vg '
                         /^Disk:/   { disk = $2 }
                         /^dgname:/ { if (vg == $2) { print disk; } }')
        list_of_devices=`echo ${list_of_devices} | sed 's/\n/ /g'`
        unset xpca_list_of_devices
        for device in ${list_of_devices}; do
            # get the persistent dsf equivalent only if the device is in legacy dsf
            if [[ $device = c*t*d* ]]; then
                #get the controller id.
                controller_id=`echo ${device} | sed 's/c\(.*\)t\(.*\)/\1/'`
                if [ ${controller_id} -gt 512 ]; then
                    #VxVM is using  a internal format to identify the disks
                    #cannot proceed with the check if the disk are in this format
                    continue
                else
                    disk_path="/dev/rdsk/"${device}
                    disk_in_pers_dsf=`${IOSCAN} -m dsf ${disk_path} | grep ${disk_path} | awk '{ print $1 }'`
                fi
            elif [[ $device = disk* ]]; then
                disk_in_pers_dsf="/dev/rdisk/"${device}
            else
                #the devices are in enclosure based format
                continue 
            fi

            # call the appropriate function based on the module used.
            checkifmodused "dts/srdf"
            if [ $? -eq 0 ]; then
                srdf_get_disk_info ${disk_in_pers_dsf} ${vg} "vxvm"
            fi

            checkifmodused "dts/xpca"
            if [ $? -eq 0 ]; then
                xpca_list_of_devices[${#xpca_list_of_devices[@]}]=$disk_in_pers_dsf
            fi

            checkifmodused "dts/caeva"
            if [ $? -eq 0 ]; then
                caeva_get_disk_info ${disk_in_pers_dsf} ${vg} "vxvm"
            fi
        done
            checkifmodused "dts/xpca"
            if [ $? -eq 0 ]; then
                xpca_get_disk_info ${vg} "vxvm"
    fi
    done
}

##################################################################################
# This function checks whether the disk passed as an argument is being replicated
# by EVA.
#
# Internal Use only
#
#
# Arguments: Disk Path , LVM/VXVM groupname, lvm/vxvm
#
#
# Returns:  None
#
###############################################################################
function caeva_get_disk_info
{
    disk_in_pers_dsf=$1
    vg=$2

    if [[ "$3" = "lvm" ]]; then
            type_of_dg="volume group"
        else
            type_of_dg="disk group"
        fi

    if [[ -f /usr/sbin/evainfo ]];then
        device_wwn=`/usr/sbin/evainfo -d ${disk_in_pers_dsf} | grep ${disk_in_pers_dsf} | awk '{print $3}' 2>/dev/null `
        if [[ "${SG_COMMAND}" = "cmcheckconf" && -z "${SG_PRIV_CLUSTER_VERIFICATION}" ]]; then
        envFile=`echo "${CAEVA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_caeva.env.internal"`
        else
        envFile=`echo "${CAEVA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_caeva.env"`
        fi
        if [[ "$SG_COMMAND" = "cmcheckconf" ]]; then
                cmdVal="cmcheckconf"
        else
                cmdVal="cmapplyconf"
        fi

        smisFile=`echo "$envFile.smis"`
        cat $envFile | grep ^[^#] | grep SMIS_LIST | awk -F[=] '{print $2}' >> $smisFile

        if [[ -z $device_wwn ]];then
            warnings[$warncount]="Warning: Disk ${disk_in_pers_dsf} in the volume group ${vg} doesn't belong to EVA Array"
            (( warncount=warncount+1 ))
        else
            retVal=0
            cmd_output=`${CMCLUSTER}/bin/mccaeva/DREVAGetDiskInfo  ${device_wwn} $envFile $cmdVal`
            if [[ $? -ne 0 ]]; then
                retVal=1
            else
                dg_name=`echo $cmd_output | awk '{print $1}'`
                if [[ -z ${dg_name} ]]; then
                    retVal=1
                fi
            fi

            if [[ $retVal -ne 0 ]]; then
                warnings[$warncount]="\n$cmd_output\nWarning: Unable to find the Data Replication group for the disk \
                    ${disk_in_pers_dsf} belonging to ${type_of_dg} ${vg}. \
                                        \nDisk ${disk_in_pers_dsf} is not a part of the device group ${CAEVA_DR_GROUP_NAME} \
                                        configured in this Metrocluster package. \nIgnore this warning if this \
                                        is being done intentionally; \nElse add this disk to the device group ${CAEVA_DR_GROUP_NAME}\n"
                                (( warncount=warncount+1 ))
            else
                if [[ ${dg_name} != ${CAEVA_DR_GROUP_NAME} ]]; then
                    if [[ ${internal_use} -eq 1 ]]; then
                        errors[$errcount]="\nError: Disk ${disk_in_pers_dsf} in the ${type_of_dg} \
                        ${vg} belongs to a different device group ${dg_name} \
                        \ninstead of the ${CAEVA_DR_GROUP_NAME} that is mentioned in the package configuration file.\n"
                    else
                        errors[$errcount]="\nError: Disk ${disk_in_pers_dsf} in the ${type_of_dg} \
                            ${vg} belongs to a different device group ${dg_name} \
                            \ninstead of the ${CAEVA_DR_GROUP_NAME} that is mentioned in the package \
                            environment file ${CAEVA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_caeva.env.\n"
                    fi
                    (( errcount=errcount+1 ))
                fi
            fi
        fi    
    else
        warnings[$warncount]="Warning: The utility /usr/sbin/evainfo is not found."
        (( warncount=warncount+1 ))
    fi

}

###############################################################################
# This function checks whether ccxxx module is included or not
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - ccxxx module is used
#          1 - ccxxx module is not used
#
###############################################################################

function check_cc_module
{
        let exit_val=1
        typeset module_name;
        typeset module;

        module_name=`echo "${SG_MODULE_NAME[@]}"`

        for module in $module_name
        do
            if [[ "$module" = "dts/ccxpca" || "$module" = "dts/cccaeva" || \
                       "$module" = "dts/ccsrdf" ]]; then
                let exit_val=0
                break
            fi
        done

    return exit_val
}

##################################################################################
# This function returns the list of nodes part of current package
#
# Internal Use only
#
# Arguments: None
#
# Returns: node list
#
###############################################################################

function get_package_nodes
{
    if (( ${#SG_NODE_NAME[*]} == 1 )); then
        if [ "${SG_NODE_NAME[0]}" == "*" ]; then
            # All the cluster nodes
            package_node_names=`echo "$cmviewcl_output_mc" | awk '/^node:[^|]*\|name=/' | cut -d "=" -f 2`
        else
            # Only one node
            package_node_names=`echo "${SG_NODE_NAME[0]}"`
        fi
    else
        # Subset of cluster nodes
        package_node_names=`echo "${SG_NODE_NAME[@]}"`
    fi
    echo "$package_node_names"
}

##################################################################################
# This function returns the UUID of node name passed.
#
# Internal Use only
#
# Arguments: node name
#
# Returns: node list
#
###############################################################################

function get_uuid_from_node_name
{
    uuid=""
    node_name=$1
    uuid=`echo "$cmviewcl_output_mc" | grep $node_name | awk -F= '/^node:[^|]*\|uuid=/ {print $2}'`
    echo $uuid
}
	
